'''
Created on Jan 31, 2014

@author: arnon
'''
import os
import re
import collections
from copy import copy
from acrilib import FlatDict
from pprint import pprint

_varprog = re.compile(r'\$(\w+|(\{[^\}]*})|(\[[^]]*\]))')
_varprogb = None


def expandvars(map_, environ=None):
    """Expand variables in flat dictionary.

    Args:
        source: flat dictionary
        environ: environment dictionary; defaults for os.environ

    Arlgorithm:
        expands ${} from environment
        expands $[] from flat names; e.g. $[EVENTOR.LOGGING.PATH]

    Returns:
        New flat dict with expanded vars.
    """
    nmap_ = type(map_)()
    if environ is None:
        environ = os.environ

    smap = find_iner_dependencies(map_)

    for k, v in list(smap.items()):
        ev = expandvar(v, environ, nmap_)
        nmap_[k] = ev

    return nmap_


def calculate_ranks(depends, rank):
    def calc_rank(k, d):
        if len(d) == 0:
            result = 0
        elif rank[k] > 0:
            result = rank[k]
        else:
            result = sum([calc_rank(i, depends[i]) for i in d])
        return result

    for k, d in depends.items():
        if d and rank[k] == 0:
            rank[k] = calc_rank(k, d)
    return rank


def find_iner_dependencies(map_):
    global _varprog

    depends = dict()
    no_depends = dict()
    for k, v in list(map_.items()):
        if not isinstance(v, str):
            no_depends[k] = v
            continue
        matchiter = _varprog.finditer(v)
        names = list()
        for m in matchiter:
            name = m.group(0)[1:]
            if name[0] == '[':
                names.append(name[1:-1])
        depends[k] = names

    rank = calculate_ranks(depends, dict([(k, 0) for k in depends.keys()]))

    combined = zip(sorted(rank.items(), key=lambda x: x[0]), sorted(depends.items(), key=lambda x: x[0]))
    combined = sorted(combined, key=lambda x: x[0][1])
    map_ = collections.OrderedDict([(rank[0], map_[rank[0]]) for rank, dependes in combined])
    map_.update(no_depends)
    return map_


def expandvar(source, environ=None, map_=None):
    """Expand shell variables of form $var and ${var}.  Unknown variables
    are left unchanged.

    Args:
        source: flat dictionary
        environ: environment dictionary; defaults for os.environ

    Arlgorithm:
        expands ${} from environment
        expands $[] from flat names; e.g. $[EVENTOR.LOGGING.PATH]

    Returns:
        New flat dict with expanded vars.
    """
    global _varprog

    if source is None or not isinstance(source, str):
        return source

    if environ is None:
        environ = os.environ
        # os.path.expandvars(path)

    if isinstance(source, bytes):
        source = source.decode()

    if source.startswith('~') or source.startswith('/~'):
        source = os.path.expanduser(source)

    if '$' not in source:
        return source

    matchiter = _varprog.finditer(source)

    for m in matchiter:
        start, end = m.span(0)
        name = m.group(0)[1:]
        if name[0] in ['{', '[']:
            varname = name[1:-1]
            typename = name[0]
        else:
            varname = name
            typename = '{'

        if typename == '{':
            env = environ
        else:
            env = map_

        try:
            value = str(env[varname])
        except KeyError:
            pass
        else:
            source = source[:start] + value + source[end:]

    return source


def pathhasvars(source):
    """Expand shell variables of form $var and ${var}.  Unknown variables
    are left unchanged."""
    global _varprog, _varprogb
    if isinstance(source, bytes):
        if b'$' not in source:
            return False
        if not _varprogb:
            import re
            # _varprogb = re.compile(br'\$(\w+|\{[^}]*\})', re.ASCII)
            _varprogb = re.compile(br'\$(\w+|\{[^}]*\})')
        search = _varprogb.search
        start = b'{'
        end = b'}'
    else:
        if '$' not in source:
            return False
        if not _varprog:
            import re
            # _varprog = re.compile(r'\$(\w+|\{[^}]*\})', re.ASCII)
            _varprog = re.compile(r'\$(\w+|\{[^}]*\})')
        search = _varprog.search
        start = '{'
        end = '}'
    i = 0
    m = search(source, i)
    return not (not m)


def expandmap(map_, expand_map=None):
    ''' Expands path variables ${}
    '''
    if not expand_map:
        expand_map = copy(os.environ)
    else:
        expand_map = copy(expand_map)

    flat = FlatDict()(map_)

    fmap = expandvars(flat)

    nmap = fmap.unflat()
    return nmap


if __name__ == '__main__':
    from pprint import pprint

    print(expandvar("~/log/environ"))

    config = {
        'workdir': '/tmp',
        'debug': False,
        'task_construct': 'process',  # or 'thread'
        'envvar_prefix': 'EVENTOR_',
        'max_concurrent': -1,
        'stop_on_exception': True,
        'sleep_between_loops': 0.25,
        'sequence_arg_name': None,  # 'eventor_task_sequence'
        'day_to_keep_db': 5,
        'remote_method': 'ssh',
        'pass_logger_to_task': False,
        'shared_db': False,
        'ssh_config': os.path.expanduser('~/.ssh/config'),
        'ssh_host': 'get_hostname()',
        'ssh_port': 22,
        'DATABASES': {'dialect': 'sqlite', 'query': {'cache': 'shared'}},
        'LOGGING': {
            'logdir': os.path.expanduser('~/log/eventor'),
            'datefmt': '%Y-%m-%d,%H:%M:%S.%f',
            'logging_level': 'logging.INFO',
            'level_formats': {
                'logging.DEBUG': ("[ %(asctime)-15s ][ %(host)s ][ %(processName)-11s ]"
                                  "[ %(levelname)-7s ][ %(message)s ]"
                                  "[ %(module)s.%(funcName)s(%(lineno)d) ]"),
                'default': ("[ %(asctime)-15s ][ %(host)s ][ %(processName)-11s ]"
                            "[ %(levelname)-7s ][ %(message)s ]"),
                },
            'consolidate': False,
            'console': True,
            'file_prefix': None,
            'file_suffix': None,
            'file_mode': 'a',
            'maxBytes': 0,
            'backupCount': 0,
            'encoding': 'utf8',
            'delay': False,
            'when': 'h',
            'interval': 1,
            'utc': False,
            'atTime': 86400,  # number of seconds in a day
        },
    }

    pprint(expandmap(config))


# -*- encoding: utf-8 -*-
##############################################################################
#
#    Acrisel LTD
#    Copyright (C) 2008- Acrisel (acrisel.com) . All Rights Reserved
#
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see http://www.gnu.org/licenses/.
#
##############################################################################
'''
Created on Oct 11, 2017

@author: arnon
'''

class LogSim(object):
    def __init__(self, file=None):
        self.file = file
        if file:
            self.file = open(file, 'w')
    
    def debug(self, msg):
        self._print('debug', msg)
    
    def info(self, msg):
        self._print('info', msg)
    
    def warning(self, msg):
        self._print('warning', msg)
    
    def error(self, msg):
        self._print('error', msg)
    
    def critical(self, msg):
        self._print('critical', msg)
        
    def _print(self, key, msg):
        print("{}: {}".format(key, msg), file=self.file)
        
    def close(self):
        if self.file:
            self.file.close()
            
    def __del__(self):
        self.close()




'''
Created on Nov 16, 2017

@author: arnon
'''
import re

text = """
# -*- encoding: utf-8 -*-
##############################################################################
#
#    Acrisel LTD
#    Copyright (C) 2008- Acrisel (acrisel.com) . All Rights Reserved
#
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see http://www.gnu.org/licenses/.
#
##############################################################################

from .idioms.decorate import traced_method, LogCaller
from .idioms.threaded import Threaded, RetriveAsycValue, threaded
from .idioms.singleton import Singleton, NamedSingleton
from .idioms.sequence import Sequence
from .idioms.data_types import MergedChainedDict
from .idioms.synchronize import Synchronization, SynchronizeAll, dont_synchronize, do_synchronize, synchronized
from .idioms.mediator import Mediator

from .idioms.osutil import get_free_port, get_ip_address, get_hostname, hostname_resolves

from .logging.formatters import LoggerAddHostFilter, LevelBasedFormatter, MicrosecondsDatetimeFormatter
from .logging.handlers import TimedSizedRotatingHandler, HierarchicalTimedSizedRotatingHandler, get_file_handler
from .logging.logsim import LogSim

from .setup.setup_utils import read_authors, read_meta_or_file, read_version
from .setup.setup_utils import is_overlay, find_packages, read, find_meta

__version__ = '1.0.4'
"""

tripple = r'^__version__[ ]*=[ ]*((\'\'\'(?P<text1>(.*\n*)*?)\'\'\')|("""(?P<text2>(.*\n*)*?)"""))'
single = '^__version__[ ]*=[ ]*((\'(?P<text1>(.*\n*)*?)\')|("(?P<text2>(.*\n*)*?)"))'

re_meta_tripple = re.compile(tripple, re.M)
re_meta_single = re.compile(single, re.M)

tmatch = re_meta_tripple.search(text)
smatch = re_meta_single.search(text)

print(tmatch)
print(smatch)



import sys
from setuptools import setup
import importlib.util


'''
is the Python package in your project. It's the top-level folder containing the
__init__.py module that should be in the same directory as your setup.py file
/-
  |- README.rst
  |- CHANGES
  |- AUTHORS
  |- LICENSE
  |- setup.py
  |- package-name
     |- __init__.py
     |- some-program.py

To create package and upload:

  python setup.py register
  python setup.py sdist
  twine upload -s dist/path/to/gz
  
  or:
  
  python setup.py sdist bdist_wheel
  twine register dist/project_name-x.y.z.tar.gz
  twine register dist/mypkg-0.1-py2.py3-none-any.whl
  twine upload -s dist/*
'''


def import_setup_utils():
    # load setup utils
    try:
        setup_utils_spec = \
            importlib.util.spec_from_file_location("setup.utils",
                                                   "setup_utils.py")
        setup_utils = importlib.util.module_from_spec(setup_utils_spec)
        setup_utils_spec.loader.exec_module(setup_utils)
    except Exception as err:
        raise RuntimeError("Failed to find setup_utils.py."
                           " Please copy or link.") from err
    return setup_utils


setup_utils = import_setup_utils()
PACKAGE = "acrilib"
NAME = PACKAGE
DESCRIPTION = ('acrilib is a python library of programming patterns'
               ' commonly used in Python')
metahost = setup_utils.metahost(PACKAGE)
AUTHORS, AUTHOR_EMAILS = setup_utils.read_authors(metahost=metahost)
URL = 'https://github.com/Acrisel/acrilib'
VERSION = setup_utils.read_version(metahost=metahost)

# Find previous installation and warn. See #18115.
existing_path = setup_utils.is_overlay(PACKAGE)

scripts = setup_utils.scripts(PACKAGE)

# Find all sub packages
packages = setup_utils.packages(PACKAGE)

setup_info = {
    'name': NAME,
    'version': VERSION,
    'url': URL,
    'author': AUTHORS,
    'author_email': AUTHOR_EMAILS,
    'description': DESCRIPTION,
    'long_description': open("README.rst", "r").read(),
    'license': 'MIT',
    'keywords': ('library sequence logger yield singleton thread synchronize'
                 ' resource pool utilities os ssh xml excel mail'),
    'packages': packages,
    'scripts': scripts,
    'install_requires': [],
    'extras_require': {'dev': [], 'test': []},
    'classifiers': [
        'Development Status :: 5 - Production/Stable',
        'Environment :: Other Environment',
        # 'Framework :: Project Settings and Operation',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Software Development :: Libraries :: Application '
        'Frameworks',
        'Topic :: Software Development :: Libraries :: Python '
        'Modules']}

setup(**setup_info)


if existing_path:
    sys.stderr.write("""

========
WARNING!
========

You have just installed %(name)s over top of an existing
installation, without removing it first. Because of this,
your install may now include extraneous files from a
previous version that have since been removed from
Accord. This is known to cause a variety of problems. You
should manually remove the

%(existing_path)s

directory and re-install %(name)s.

""" % {"existing_path": existing_path,
       "name": NAME, })



# -*- encoding: utf-8 -*-
##############################################################################
#
#    Acrisel LTD
#    Copyright (C) 2008- Acrisel (acrisel.com) . All Rights Reserved
#
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see http://www.gnu.org/licenses/.
#
##############################################################################

from .idioms.decorate import traced_method, LogCaller
from .idioms.threaded import Threaded, RetriveAsycValue, threaded
from .idioms.singleton import Singleton, NamedSingleton
from .idioms.sequence import Sequence
from .idioms.data_types import MergedChainedDict
from .idioms.synchronize import Synchronization, SynchronizeAll, dont_synchronize, do_synchronize, synchronized
from .idioms.mediator import Mediator
from .idioms.complex_mapping import FlatDict
from .idioms.expandvars import expandmap

from .idioms.osutil import get_free_port, get_ip_address, get_hostname, hostname_resolves

from .logging.formatters import LoggerAddHostFilter, LevelBasedFormatter, MicrosecondsDatetimeFormatter
from .logging.handlers import TimedSizedRotatingHandler, HierarchicalTimedSizedRotatingHandler, get_file_handler
from .logging.logsim import LogSim

__version__='1.0.5'
